package utils

import (
	"math"

	"pluja.dev/kycnot.me/database"
)

func ComputeScore(s *database.Service) string {
	const (
		baseScore     = 100
		goodRating    = 0
		warningRating = 10
		badRating     = 20
		maxScore      = 100
		minScore      = 0
		bonusLow      = 15
		bonusMedium   = 20
		bonusHigh     = 25
	)

	grade := float64(baseScore)

	// Attribute Ratings
	for _, attr := range s.Expand["attributes"] {
		switch attr.Rating {
		case "warn":
			grade -= bonusLow
		case "bad":
			grade -= bonusHigh
		}
	}

	// Tos Highlights Penalty
	if s.TosReviews == nil {
		s.TosReviews = []database.TosReview{}
	}

	trPenalty := 0
	for _, tr := range s.TosReviews {
		if tr.Warning {
			trPenalty -= 2
		}
	}

	if trPenalty > bonusHigh {
		trPenalty = bonusHigh
	}
	grade += float64(trPenalty)

	// Cash/Monero Bonus
	if s.Cash || s.Xmr {
		grade += bonusLow
	}

	// KYC Level Adjustment
	switch s.KycLevel {
	case 1:
		grade -= 5
	case 2:
		grade -= 10
	case 3:
		grade -= 15
	}

	// TODO: Manage bonuses
	// P2P/OpenSource Bonus
	/*if strings.Contains(s.Attributes, database.AttributeP2P) || strings.Contains(s.Attributes, database.AttributeOpenSource) {
		grade += bonusLow
	}*/

	// Tor URL Bonus
	if len(s.OnionUrls) == 0 || s.OnionUrls[0] == "" {
		grade -= bonusLow
	}

	// Normalize the grade to be within 0-10 bounds
	grade = math.Min(maxScore, math.Max(minScore, grade))

	switch {
	case grade >= 95:
		return "A"
	case grade >= 85:
		return "A"
	case grade >= 75:
		return "B"
	case grade >= 65:
		return "B"
	case grade >= 55:
		return "C"
	case grade >= 45:
		return "C"
	case grade >= 35:
		return "D"
	default:
		return "F"
	}
}

/*func UpdateScore(s *ent.Service) error {
	score := ComputeScore(s)
	ctx := context.Background()
	_, err := database.Client.Service.UpdateOneID(s.ID).SetScore(score).Save(ctx)
	return err
}*/
